#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <strings.h>

#define BUF_SIZE 320*200
char buf[BUF_SIZE];
unsigned char pal[256][3];

#pragma pack (1)
typedef struct {
  char cID;
  int  cSize;
} MOVCHUNK;

typedef struct {
  unsigned char IDLength;
  unsigned char ColorMapType;
  unsigned char ImageType;
  unsigned short CMapStart;
  unsigned short CMapLength;
  unsigned char CMapDepth;
  unsigned short XOffset;
  unsigned short YOffset;
  unsigned short Width;
  unsigned short Height;
  unsigned char PixelDepth;
  unsigned char ImageDescriptor;
} tga_head;

typedef struct {
  char riff[4];
  int  flsz;
  char wave[8];
  int  wvsz;
  short int pcmf;
  short int chnl;
  int  sdhz;
  int  bpsc;
  short int bpsm;
  short int bits;
  char data[4];
  int  dtsz;
} wav_head;

int f_num = 0;

void dumpthis(){
FILE *f;
char s[32];
tga_head th;
char img[BUF_SIZE];
int x,y;
  memset(&th, 0, sizeof(th));
  th.Width = 320;
  th.Height = 200;
  th.ColorMapType = 1;
  th.ImageType = 1;
  th.CMapLength = 256;
  th.CMapDepth = 24;
  th.PixelDepth = 8;
  th.ImageDescriptor = 0x20;

  /* rotate image 90* */
  for(y = 0; y < 200; y++){
    for(x = 0; x < 320; x++){
      img[(y*320) + x] = buf[(x*200) + y];
    }
  }

  sprintf(s, "%08d.tga", f_num);
  f = fopen(s, "wb");
  fwrite(&th, 1, sizeof(th), f);
  fwrite(&pal[0][0], 1, 256*3, f);
  fwrite(&img[0], 1, BUF_SIZE, f);
  fclose(f);

  f_num++;
}

int main(int argc, char *argv[]){
FILE *fl, *fa;
char *p;
short int i;
unsigned char a, b;
int flsize;
MOVCHUNK mc;
wav_head wh;
  fl = fopen("BOOK.MOV", "rb");
  if(fl == NULL){
    printf("error opening BOOK.MOV file\n");
    return(1);
  }
  fseek(fl, 0, SEEK_END);
  flsize = ftell(fl);
  fseek(fl, 32, SEEK_SET);
  /* init */
  fa = fopen("BOOK.WAV", "wb");
  memset(&wh, 0, sizeof(wh));
  fwrite(&wh, 1, sizeof(wh), fa);
  /* decode */
  while(ftell(fl) < flsize){
    fread(&mc, sizeof(mc), 1, fl);
    switch(mc.cID - 1){
      case 0: /* PALETTE */
        fread(&pal[0][0], 1, 256*3, fl);
        mc.cSize -= 256*3;
        fread(&b, 1, 1, fl);
        mc.cSize --;                  /* here mc.cSize must be zero */
        memset(&buf[0], b, BUF_SIZE); /* fill buffer with initial color */
        for(i = 0; i < 256; i++){
          /* VGA [0..63] -> RGB [0..255] */
          for(a = 0; a < 3; a++){
            pal[i][a] = ((pal[i][a] << 2) | (pal[i][a] >> 4));
          }
          /* RGB -> BGR */
          pal[i][0] ^= pal[i][2];
          pal[i][2] ^= pal[i][0];
          pal[i][0] ^= pal[i][2];
        }
        break;
      case 1: /* AUDIO */
        p = (char *)malloc(mc.cSize);
        fread(p, 1, mc.cSize, fl);
        fwrite(p, 1, mc.cSize, fa);
        free(p);
        break;
      case 2: /* VIDEO */
        fread(&i, 2, 1, fl);
        mc.cSize -= 2;
        p = &buf[0];   /* image start offs */ 
        p += (200*i);
        while(mc.cSize > 0){
          fread(&a, 1, 1, fl);
          fread(&b, 1, 1, fl);
          mc.cSize -= 2;
          p += a;
          if(b){
            fread(p, 1, b, fl);
            mc.cSize -= b;
            p += b;
          }
        }
        dumpthis();
        break;
      case 3: /* SYNC */
        fseek(fl, mc.cSize, SEEK_CUR);
        dumpthis();
        break;
      default:
        fseek(fl, mc.cSize, SEEK_CUR);
        break;
    }
  }
  /* fix wave header */
  strncpy(wh.riff, "RIFF", 4);
  strncpy(wh.data, "data", 4);
  strncpy(wh.wave, "WAVEfmt ", 8);
  wh.wvsz = 16;
  wh.pcmf = 1;
  wh.bits = 8;
  wh.chnl = 1;
  wh.sdhz = 22050; /* or 22222 ? or maybe this stored in .MOV header? */
  wh.bpsm = (wh.bits/8) * wh.chnl;
  wh.bpsc = wh.sdhz * wh.bpsm;
  wh.dtsz = ftell(fa) - sizeof(wh);
  wh.flsz = wh.dtsz + sizeof(wh) - 8;
  fseek(fa, 0, SEEK_SET);
  fwrite(&wh, 1, sizeof(wh), fa);
  fclose(fa);
  fclose(fl);
  return(0);
}
